﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WWWCFilterEditor
{
	public partial class MainForm : Form
	{
		List<Item> items;

		private enum Operation
		{
			Delete,
			Strip
		}

		private class Item
		{
			public Operation operation;
			public string url;
			public string start;
			public string end;
			public bool repeat;
			public bool empty;
			public bool ignore;
			public bool ignoreCase;
			public bool comment;

			public Item(Operation operation, bool repeat, bool empty, bool ignore, bool ignoreCase, bool comment, string url, string start, string end)
			{
				this.operation = operation;
				this.repeat = repeat;
				this.empty = empty;
				this.ignore = ignore;
				this.ignoreCase = ignoreCase;
				this.comment = comment;
				this.url = url;
				this.start = start;
				this.end = end;
			}

			public Item(Operation operation, string url, string start, string end) :this(operation, false, false, false, false, false, url, start, end)
			{
			}

			public static Item CreateItemFromFilter(string filter)
			{
				if (string.IsNullOrEmpty(filter))
					return null;
				Item result = new Item(Operation.Delete, "", "", "");
				string[] parts = filter.Split('\t');
				if (parts.Length < 4)
					return null;

				for (int i = 0; i < parts[0].Length; i++)
				{
					if (parts[0][i] == 'D')
						result.operation = Operation.Delete;
					else if (parts[0][i] == 'S')
						result.operation = Operation.Strip;
					else if (parts[0][i] == 'R')
						result.repeat = true;
					else if (parts[0][i] == 'E')
						result.empty = true;
					else if (parts[0][i] == 'I')
						result.ignore = true;
					else if (parts[0][i] == 'C')
						result.ignoreCase = true;
					else if (parts[0][i] == '#')
						result.comment = true;
				}

				if (result.repeat && result.operation != Operation.Delete)
					result.repeat = false;
				if (result.empty && result.operation != Operation.Strip)
					result.empty = false;

				result.url = parts[1];
				result.start = parts[2];
				result.end = parts[3];

				return result;
			}

			public string ConvertToFilter()
			{
				StringBuilder builder = new StringBuilder();

				if (operation == Operation.Strip)
				{
					builder.Append("S");
					if (empty)
						builder.Append("E");
				}
				else
				{
					builder.Append("D");
					if (repeat)
						builder.Append("R");
				}

				if (ignore)
					builder.Append("I");

				if (ignoreCase)
					builder.Append("C");

				if (comment)
					builder.Insert(0, "#");

				builder.Append("\t" + url + "\t" + start + "\t" + end);

				return builder.ToString();
			}

			public override string ToString()
			{
				return url;
			}
		}

		public MainForm()
		{
			InitializeComponent();
			items = new List<Item>();
		}

		private void ClearForm()
		{
			urlTextBox.Text = "";
			startTextBox.Text = "";
			endTextBox.Text = "";
			deleteRadioButton.Checked = true;
			repeatCheckBox.Checked = false;
			emptyCheckBox.Checked = false;
			ignoreCheckBox.Checked = false;
			ignoreCaseCheckBox.Checked = false;
			commentCheckBox.Checked = false;
		}

		private void CreateForm(Item item)
		{
			if (item.operation == Operation.Delete)
				deleteRadioButton.Checked = true;
			else
				stripRadioButton.Checked = true;

			repeatCheckBox.Checked = item.repeat;
			emptyCheckBox.Checked = item.empty;
			ignoreCheckBox.Checked = item.ignore;
			ignoreCaseCheckBox.Checked = item.ignoreCase;
			commentCheckBox.Checked = item.comment;
			urlTextBox.Text = item.url;
			startTextBox.Text = item.start;
			endTextBox.Text = item.end;
		}

		private void addButton_Click(object sender, EventArgs e)
		{
			Operation ope = stripRadioButton.Checked ? Operation.Strip : Operation.Delete;
			string url = string.IsNullOrEmpty(urlTextBox.Text) ? "http://" : urlTextBox.Text;
			Item newItem = new Item(ope,
				repeatCheckBox.Checked,
				emptyCheckBox.Checked,
				ignoreCheckBox.Checked,
				ignoreCaseCheckBox.Checked,
				commentCheckBox.Checked,
				url,
				startTextBox.Text,
				endTextBox.Text);
			items.Add(newItem);
			listBox.Items.Add(newItem);
		}

		private void listBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (listBox.SelectedIndex == -1)
				return;

			ClearForm();
			Item selectedItem = (Item)listBox.Items[listBox.SelectedIndex];
			CreateForm(selectedItem);
		}

		private void editButton_Click(object sender, EventArgs e)
		{
			if (listBox.SelectedItem == null)
				return;

			Item selectedItem = (Item)listBox.SelectedItem;
			if (deleteRadioButton.Checked)
				selectedItem.operation = Operation.Delete;
			else if (stripRadioButton.Checked)
				selectedItem.operation = Operation.Strip;

			selectedItem.repeat = repeatCheckBox.Checked;
			selectedItem.empty = emptyCheckBox.Checked;
			selectedItem.ignore = ignoreCheckBox.Checked;
			selectedItem.ignoreCase = ignoreCaseCheckBox.Checked;
			selectedItem.comment = commentCheckBox.Checked;

			selectedItem.url = urlTextBox.Text;
			selectedItem.start = startTextBox.Text;
			selectedItem.end = endTextBox.Text;
		}

		private void deleteButton_Click(object sender, EventArgs e)
		{
			if (listBox.SelectedIndex == -1)
				return;
			listBox.Items.RemoveAt(listBox.SelectedIndex);
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			openFileDialog.ShowDialog(this);
			if (!File.Exists(openFileDialog.FileName))
				return;
			using (StreamReader reader = new StreamReader(openFileDialog.OpenFile(), Encoding.Default))
			{
				do
				{
					string line = reader.ReadLine();
					Item item = Item.CreateItemFromFilter(line);
					if (item == null)
						continue;
					listBox.Items.Add(Item.CreateItemFromFilter(line));
				}
				while (!reader.EndOfStream);
			}
		}

		private void saveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(saveFileDialog.FileName))
			{
				if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
				{
					using (StreamWriter writer = new StreamWriter(saveFileDialog.OpenFile(), Encoding.Default))
					{
						for (int i = 0; i < listBox.Items.Count; i++)
						{
							writer.WriteLine(((Item)listBox.Items[i]).ConvertToFilter());
						}
					}
				}
			}
			else
			{
				using (StreamWriter writer = new StreamWriter(saveFileDialog.OpenFile(), Encoding.Default))
				{
					for (int i = 0; i < listBox.Items.Count; i++)
					{
						writer.WriteLine(((Item)listBox.Items[i]).ConvertToFilter());
					}
				}
			}
		}

		private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
			{
				using (StreamWriter writer = new StreamWriter(saveFileDialog.OpenFile(), Encoding.Default))
				{
					for (int i = 0; i < listBox.Items.Count; i++)
					{
						writer.WriteLine(((Item)listBox.Items[i]).ConvertToFilter());
					}
				}
			}
		}

		private void bbsCopyToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (listBox.SelectedIndex != -1)
			{
				Clipboard.SetText(((Item)listBox.SelectedItem).ConvertToFilter().Replace("\t", "【TAB】"));
			}
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			using (StreamWriter writer = new StreamWriter("backup.txt", false, Encoding.Default))
			{
				for (int i = 0; i < listBox.Items.Count; i++)
				{
					writer.WriteLine(((Item)listBox.Items[i]).ConvertToFilter());
				}
			}
		}
	}
}
